/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JStatement.java 1022 2006-08-04 11:02:28Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.component.jdbcpool;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * Wrapper on a PreparedStatement. This wrapper is used to track close method in
 * order to avoid closing the statement, and putting it instead in a pool.
 * @author Philippe Durieux
 * @author Florent Benoit
 */
public class JStatement implements PreparedStatement {

    /**
     * Properties of this statement has been changed ? Needs to be be cleared when reused.
     */
    private boolean changed = false;

    /**
     * Is that this statement is opened ?
     */
    private boolean opened = false;

    /**
     * Being closed. (in close method).
     */
    private boolean closing = false;

    /**
     * Physical PreparedStatement object on which the wrapper is.
     */
    private PreparedStatement ps;

    /**
     * Managed Connection the Statement belongs to.
     */
    private JManagedConnection mc;

    /**
     * Hashcode computed in constructor.
     */
    private int hashCode;

    /**
     * SQL used as statement.
     */
    private String sql;

    /**
     * Logger.
     */
    private JLog logger = JLogFactory.getLog(JStatement.class);

    /**
     * Builds a new statement with the given wrapped statement of given connection and given sql query.
     * @param ps the prepared statement.
     * @param mc managed connection
     * @param sql query.
     */
    public JStatement(final PreparedStatement ps, final JManagedConnection mc, final String sql) {
        this.ps = ps;
        this.mc = mc;
        this.sql = sql;
        hashCode = sql.hashCode();
        opened = true;
    }

    /**
     * @return Sql query used.
     */
    public String getSql() {
        return sql;
    }

    /**
     * @return hashcode of the object
     */
    @Override
    public int hashCode() {
        return hashCode;
    }

    /**
     * @param stmt given statement for comparing it
     * @return true if given object is equals to this current object
     */
    @Override
    public boolean equals(final Object stmt) {
        if (stmt == null) {
            return false;
        }
        // different hashcode, cannot be equals
        if (this.hashCode != stmt.hashCode()) {
            return false;
        }

        // if got same hashcode, try to see if cast is ok.
        if (!(stmt instanceof JStatement)) {
            logger.warn("Bad class {0}", stmt);
            return false;
        }
        JStatement psw = (JStatement) stmt;
        if (sql == null && psw.getSql() != null) {
            return false;
        }
        if (sql != null && !sql.equals(psw.getSql())) {
            return false;
        }
        try {
            if (psw.getResultSetType() != getResultSetType()) {
                return false;
            }
            if (psw.getResultSetConcurrency() != getResultSetConcurrency()) {
                return false;
            }
        } catch (SQLException e) {
            logger.warn("Cannot compare statements", e);
            return false;
        }
        logger.debug("Found");
        return true;
    }

    /**
     * Force a close on the Prepare Statement. Usually, it's the caller that did
     * not close it explicitly
     * @return true if it was open
     */
    public boolean forceClose() {
        if (opened) {
            logger.debug("Statements should be closed explicitly.");
            opened = false;
            return true;
        }
        return false;
    }

    /**
     * Reuses this statement so reset properties.
     * @throws SQLException if reset fails
     */
    public void reuse() throws SQLException {
        ps.clearParameters();
        ps.clearWarnings();
        opened = true;
        if (changed) {
            logger.debug("Properties statement have been changed, reset default properties");
            ps.clearBatch();
            ps.setFetchDirection(ResultSet.FETCH_FORWARD);
            ps.setMaxFieldSize(0);
            ps.setMaxRows(0);
            ps.setQueryTimeout(0);
            changed = false;
        }
    }

    /**
     * @return true if this statement has been closed, else false.
     */
    public boolean isClosed() {
        return !opened && !closing;
    }

    /**
     * Physically close this Statement.
     * @throws SQLException
     */
    public void forget() {
        try {
            ps.close();
        } catch (SQLException e) {
            logger.error("Cannot close the PreparedStatement", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public int executeUpdate() throws SQLException {
        return ps.executeUpdate();
    }

    /**
     * {@inheritDoc}
     */
    public void addBatch() throws SQLException {
        changed = true;
        ps.addBatch();
    }

    /**
     * {@inheritDoc}
     */
    public void clearParameters() throws SQLException {
        ps.clearParameters();
    }

    /**
     * {@inheritDoc}
     */
    public boolean execute() throws SQLException {
        return ps.execute();
    }

    /**
     * {@inheritDoc}
     */
    public void setByte(final int parameterIndex, final byte x) throws SQLException {
        ps.setByte(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setDouble(final int parameterIndex, final double x) throws SQLException {
        ps.setDouble(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setFloat(final int parameterIndex, final float x) throws SQLException {
        ps.setFloat(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setInt(final int parameterIndex, final int x) throws SQLException {
        ps.setInt(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setNull(final int parameterIndex, final int sqlType) throws SQLException {
        ps.setNull(parameterIndex, sqlType);
    }

    /**
     * {@inheritDoc}
     */
    public void setLong(final int parameterIndex, final long x) throws SQLException {
        ps.setLong(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setShort(final int parameterIndex, final short x) throws SQLException {
        ps.setShort(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setBoolean(final int parameterIndex, final boolean x) throws SQLException {
        ps.setBoolean(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setBytes(final int parameterIndex, final byte[] x) throws SQLException {
        ps.setBytes(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setAsciiStream(final int parameterIndex, final InputStream x, final int length) throws SQLException {
        ps.setAsciiStream(parameterIndex, x, length);
    }

    /**
     * {@inheritDoc}
     */
    public void setBinaryStream(final int parameterIndex, final InputStream x, final int length) throws SQLException {
        ps.setBinaryStream(parameterIndex, x, length);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("deprecation")
    public void setUnicodeStream(final int parameterIndex, final InputStream x, final int length) throws SQLException {
        ps.setUnicodeStream(parameterIndex, x, length);
    }

    /**
     * {@inheritDoc}
     */
    public void setCharacterStream(final int parameterIndex, final Reader reader, final int length) throws SQLException {
        ps.setCharacterStream(parameterIndex, reader, length);
    }

    /**
     * {@inheritDoc}
     */
    public void setObject(final int parameterIndex, final Object x) throws SQLException {
        ps.setObject(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setObject(final int parameterIndex, final Object x, final int targetSqlType) throws SQLException {
        ps.setObject(parameterIndex, x, targetSqlType);
    }

    /**
     * {@inheritDoc}
     */
    public void setObject(final int parameterIndex, final Object x,
            final int targetSqlType, final int scale) throws SQLException {
        ps.setObject(parameterIndex, x, targetSqlType, scale);
    }

    /**
     * {@inheritDoc}
     */
    public void setNull(final int paramIndex, final int sqlType, final String typeName) throws SQLException {
        ps.setNull(paramIndex, sqlType, typeName);
    }

    /**
     * {@inheritDoc}
     */
    public void setString(final int parameterIndex, final String x) throws SQLException {
        ps.setString(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setBigDecimal(final int parameterIndex, final BigDecimal x) throws SQLException {
        ps.setBigDecimal(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setURL(final int parameterIndex, final URL x) throws SQLException {
        ps.setURL(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setArray(final int i, final Array x) throws SQLException {
        ps.setArray(i, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setBlob(final int i, final Blob x) throws SQLException {
        ps.setBlob(i, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setClob(final int i, final Clob x) throws SQLException {
        ps.setClob(i, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setDate(final int parameterIndex, final Date x) throws SQLException {
        ps.setDate(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public ParameterMetaData getParameterMetaData() throws SQLException {
        return ps.getParameterMetaData();
    }

    /**
     * {@inheritDoc}
     */
    public void setRef(final int i, final Ref x) throws SQLException {
        ps.setRef(i, x);
    }

    /**
     * {@inheritDoc}
     */
    public ResultSet executeQuery() throws SQLException {
        return ps.executeQuery();
    }

    /**
     * {@inheritDoc}
     */
    public ResultSetMetaData getMetaData() throws SQLException {
        return ps.getMetaData();
    }

    /**
     * {@inheritDoc}
     */
    public void setTime(final int parameterIndex, final Time x) throws SQLException {
        ps.setTime(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setTimestamp(final int parameterIndex, final Timestamp x) throws SQLException {
        ps.setTimestamp(parameterIndex, x);
    }

    /**
     * {@inheritDoc}
     */
    public void setDate(final int parameterIndex, final Date x, final Calendar cal) throws SQLException {
        ps.setDate(parameterIndex, x, cal);
    }

    /**
     * {@inheritDoc}
     */
    public void setTime(final int parameterIndex, final Time x, final Calendar cal) throws SQLException {
        ps.setTime(parameterIndex, x, cal);
    }

    /**
     * {@inheritDoc}
     */
    public void setTimestamp(final int parameterIndex, final Timestamp x, final Calendar cal) throws SQLException {
        ps.setTimestamp(parameterIndex, x, cal);
    }

    /**
     * {@inheritDoc}
     */
    public int getFetchDirection() throws SQLException {
        return ps.getFetchDirection();
    }

    /**
     * {@inheritDoc}
     */
    public int getFetchSize() throws SQLException {
        return ps.getFetchSize();
    }

    /**
     * {@inheritDoc}
     */
    public int getMaxFieldSize() throws SQLException {
        return ps.getMaxFieldSize();
    }

    /**
     * {@inheritDoc}
     */
    public int getMaxRows() throws SQLException {
        return ps.getMaxRows();
    }

    /**
     * {@inheritDoc}
     */
    public int getQueryTimeout() throws SQLException {
        return ps.getQueryTimeout();
    }

    /**
     * {@inheritDoc}
     */
    public int getResultSetConcurrency() throws SQLException {
        return ps.getResultSetConcurrency();
    }

    /**
     * {@inheritDoc}
     */
    public int getResultSetHoldability() throws SQLException {
        return ps.getResultSetHoldability();
    }

    /**
     * {@inheritDoc}
     */
    public int getResultSetType() throws SQLException {
        return ps.getResultSetType();
    }

    /**
     * {@inheritDoc}
     */
    public int getUpdateCount() throws SQLException {
        return ps.getUpdateCount();
    }

    /**
     * {@inheritDoc}
     */
    public void cancel() throws SQLException {
        ps.cancel();
    }

    /**
     * {@inheritDoc}
     */
    public void clearBatch() throws SQLException {
        ps.clearBatch();
    }

    /**
     * {@inheritDoc}
     */
    public void clearWarnings() throws SQLException {
        ps.clearWarnings();
    }

    /**
     * {@inheritDoc}
     */
    public void close() throws SQLException {
        if (!opened) {
            logger.debug("Statement already closed");
            return;
        }
        opened = false;
        closing = true;
        mc.notifyPsClose(this);
        closing = false;
    }

    /**
     * {@inheritDoc}
     */
    public boolean getMoreResults() throws SQLException {
        return ps.getMoreResults();
    }

    /**
     * {@inheritDoc}
     */
    public int[] executeBatch() throws SQLException {
        return ps.executeBatch();
    }

    /**
     * {@inheritDoc}
     */
    public void setFetchDirection(final int direction) throws SQLException {
        changed = true;
        ps.setFetchDirection(direction);
    }

    /**
     * {@inheritDoc}
     */
    public void setFetchSize(final int rows) throws SQLException {
        changed = true;
        ps.setFetchSize(rows);
    }

    /**
     * {@inheritDoc}
     */
    public void setMaxFieldSize(final int max) throws SQLException {
        changed = true;
        ps.setMaxFieldSize(max);
    }

    /**
     * {@inheritDoc}
     */
    public void setMaxRows(final int max) throws SQLException {
        changed = true;
        ps.setMaxRows(max);
    }

    /**
     * {@inheritDoc}
     */
    public void setQueryTimeout(final int seconds) throws SQLException {
        changed = true;
        ps.setQueryTimeout(seconds);
    }

    /**
     * {@inheritDoc}
     */
    public boolean getMoreResults(final int current) throws SQLException {
        return ps.getMoreResults(current);
    }

    /**
     * {@inheritDoc}
     */
    public void setEscapeProcessing(final boolean enable) throws SQLException {
        ps.setEscapeProcessing(enable);
    }

    /**
     * {@inheritDoc}
     */
    public int executeUpdate(final String sql) throws SQLException {
        return ps.executeUpdate(sql);
    }

    /**
     * {@inheritDoc}
     */
    public void addBatch(final String sql) throws SQLException {
        changed = true;
        ps.addBatch(sql);
    }

    /**
     * {@inheritDoc}
     */
    public void setCursorName(final String name) throws SQLException {
        ps.setCursorName(name);
    }

    /**
     * {@inheritDoc}
     */
    public boolean execute(final String sql) throws SQLException {
        changed = true;
        return ps.execute(sql);
    }

    /**
     * {@inheritDoc}
     */
    public int executeUpdate(final String sql, final int autoGeneratedKeys) throws SQLException {
        changed = true;
        return ps.executeUpdate(sql, autoGeneratedKeys);
    }

    /**
     * {@inheritDoc}
     */
    public boolean execute(final String sql, final int autoGeneratedKeys) throws SQLException {
        changed = true;
        return ps.execute(sql, autoGeneratedKeys);
    }

    /**
     * {@inheritDoc}
     */
    public int executeUpdate(final String sql, final int[] columnIndexes) throws SQLException {
        changed = true;
        return ps.executeUpdate(sql, columnIndexes);
    }

    /**
     * {@inheritDoc}
     */
    public boolean execute(final String sql, final int[] columnIndexes) throws SQLException {
        changed = true;
        return ps.execute(sql, columnIndexes);
    }

    /**
     * {@inheritDoc}
     */
    public Connection getConnection() throws SQLException {
        return ps.getConnection();
    }

    /**
     * {@inheritDoc}
     */
    public ResultSet getGeneratedKeys() throws SQLException {
        return ps.getGeneratedKeys();
    }

    /**
     * {@inheritDoc}
     */
    public ResultSet getResultSet() throws SQLException {
        return ps.getResultSet();
    }

    /**
     * {@inheritDoc}
     */
    public SQLWarning getWarnings() throws SQLException {
        return ps.getWarnings();
    }

    /**
     * {@inheritDoc}
     */
    public int executeUpdate(final String sql, final String[] columnNames) throws SQLException {
        return ps.executeUpdate(sql, columnNames);
    }

    /**
     * {@inheritDoc}
     */
    public boolean execute(final String sql, final String[] columnNames) throws SQLException {
        return ps.execute(sql, columnNames);
    }

    /**
     * {@inheritDoc}
     */
    public ResultSet executeQuery(final String sql) throws SQLException {
        return ps.executeQuery(sql);
    }

}
