/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ComponentRegistry.java 1066 2006-08-09 17:00:56Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.component;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.objectweb.easybeans.component.api.EZBComponent;
import org.objectweb.easybeans.component.api.EZBComponentException;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * Registry that manages components. It allows to get components.
 * @author Florent Benoit
 */
public class ComponentRegistry {

    /**
     * Logger.
     */
    private JLog logger = JLogFactory.getLog(ComponentRegistry.class);

    /**
     * Map of components.<br/> Name <--> Implementation of the component
     */
    private Map<String, EZBComponent> components = null;

    /**
     * Constructor.
     */
    public ComponentRegistry() {
        // init map
        components = new HashMap<String, EZBComponent>();
    }

    /**
     * Register a component.
     * @param componentName the name of the component to register
     * @param component the component to register.
     * @throws EZBComponentException if registering fails.
     */
    public void register(final String componentName, final EZBComponent component) throws EZBComponentException {
        // Existing ?
        if (components.containsKey(componentName)) {
            throw new EZBComponentException("Cannot register the component with the name '" + componentName
                    + "'. There is an existing component with this name.");
        }

        logger.debug("Registering component with name {0}.", componentName);
        components.put(componentName, component);
    }

    /**
     * Unregister a component.
     * @param componentName the component name to unregister.
     * @throws EZBComponentException if unregistering fails.
     */
    public void unregister(final String componentName) throws EZBComponentException {
        // Exist ?
        if (!components.containsKey(componentName)) {
            throw new EZBComponentException("No component with the name '" + componentName
                    + "' found. Component not unregistered");
        }

        logger.info("Unregistering component with name {0}.", componentName);
        components.remove(componentName);
    }

    /**
     * Unregister a component.
     * @param component the instance of the component to unregister.
     * @throws EZBComponentException if unregistering fails.
     */
    public void unregister(final EZBComponent component) throws EZBComponentException {
        String name = null;

        // Find component
        Set<String> keys = components.keySet();
        for (String key : keys) {
            EZBComponent foundComponent = components.get(key);
            if (foundComponent.equals(component)) {
                // got it !
                name = key;
                break;
            }
        }
        // found --> unregister.
        if (name != null) {
            unregister(name);
        }
        throw new EZBComponentException("No component found in the registry with the given component '" + component + "'.");

    }

    /**
     * Allow to get a reference on another component.
     * @param componentName the name of the component
     * @return the component.
     */
    public EZBComponent getComponent(final String componentName) {
        return components.get(componentName);
    }

    /**
     * @param component EZBComponent instance.
     * @return Returns the component name from the EZBComponent instance.
     */
    public String getComponentName(final EZBComponent component) {

        // Iterates over the components to find the component's name
        String match = null;
        for(Iterator<String> i = this.components.keySet().iterator();
            i.hasNext() && (match == null);) {
            String key = i.next();
            EZBComponent candidate = this.components.get(key);
            if (component == candidate) {
                match = key;
            }
        }
        if (match == null) {
            throw new IllegalStateException("Each component should be registered in the registry.");
        }
        return match;
    }
}
