/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EZBJarArchiveImpl.java 1026 2006-08-04 14:54:10Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.archive;

import static org.objectweb.easybeans.util.url.URLUtils.fileToURL;
import static org.objectweb.easybeans.util.url.URLUtils.fileToURL2;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import org.objectweb.easybeans.api.EZBArchive;
import org.objectweb.easybeans.api.EZBArchiveException;
import org.objectweb.easybeans.util.url.URLUtilsException;
/**
 * Creates wrapper around jar file.
 * @author Florent Benoit
 */
public class EZBJarArchiveImpl implements EZBArchive {

    /**
     * Internal resource used as archive.
     */
    private JarFile jarFile = null;

    /**
     * Internal resource used as archive.
     */
    private File file = null;

    /**
     * Creates new instance of an EZBArchive for a jar file.
     * @param file the given jar file.
     */
    protected EZBJarArchiveImpl(final File file) {
        this.file = file;

    }

    /**
     * Init an archive and create the associated jar file object. Close() method
     * needs to be called !.
     * @throws EZBArchiveException if init fails
     */
    private void init() throws EZBArchiveException {
        try {
            jarFile = new JarFile(file);
        } catch (IOException e) {
            throw new EZBArchiveException("Invalid file", e);
        }
    }

    /**
     * @return a description of this archive. This name could be used in logger
     *         info.
     */
    public String getName() {
        return file.getPath();
    }

    /**
     * Close the underlying Resource.
     * @return Returns <code>true</code> if the close was succesful,
     *         <code>false</code> otherwise.
     */
    public boolean close() {
        // Never accessed.
        if (jarFile == null) {
            return true;
        }
        try {
            jarFile.close();
        } catch (IOException e) {
            return false;
        }
        return true;
    }

    /**
     * @param resourceName The resource name to be looked up.
     * @return Returns the resource URL if the resource has been found. null
     *         otherwise.
     * @throws EZBArchiveException if method fails.
     */
    public URL getResource(final String resourceName) throws EZBArchiveException {
        URL url = null;
        // Open jar file.
        init();
        try {
            JarEntry jarEntry = jarFile.getJarEntry(resourceName);
            if (jarEntry != null) {
                try {
                    url = new URL("jar:" + fileToURL(file) + "!/" + resourceName);
                } catch (MalformedURLException e) {
                    throw new EZBArchiveException("Invalid url", e);
                }
            }
        } finally {
            close();
        }
        return url;
    }

    /**
     * @return Returns an Iterator of Resource's URL.
     * @throws EZBArchiveException if method fails.
     */
    public Iterator<URL> getResources() throws EZBArchiveException {
        List<URL> listResources = new ArrayList<URL>();
        init();
        try {
            Enumeration<? extends ZipEntry> en = jarFile.entries();
            while (en.hasMoreElements()) {
                ZipEntry zipEntry = en.nextElement();
                String name = zipEntry.getName();
                try {
                    listResources.add(new URL("jar:" + fileToURL(file) + "!/" + name));
                } catch (MalformedURLException e) {
                    throw new EZBArchiveException("Invalid url", e);
                }
            }
        } finally {
            close();
        }
        return listResources.iterator();
    }

    /**
     * @param resourceName The resource name to be looked up.
     * @return Returns an Iterator of matching resources.
     * @throws EZBArchiveException if method fails.
     */
    public Iterator<URL> getResources(final String resourceName) throws EZBArchiveException {
        List<URL> listResources = new ArrayList<URL>();
        URL url = getResource(resourceName);
        if (url != null) {
            listResources.add(url);
        }

        return listResources.iterator();
    }

    /**
     * @return Returns the resource URL.
     * @throws EZBArchiveException if method fails.
     */
    public URL getURL() throws EZBArchiveException {
        try {
            return fileToURL2(file);
        } catch (URLUtilsException e) {
            throw new EZBArchiveException("Invalid url", e);
        }
    }

    /**
     * Is that the given object is equals to our instance.
     * @param o the object to compare.
     * @return true if equals, else false.
     */
    @Override
    public boolean equals(final Object o) {
        if (!(o instanceof EZBJarArchiveImpl)) {
            return false;
        }
        EZBJarArchiveImpl other = (EZBJarArchiveImpl) o;
        return this.file.equals(other.file);
    }

    /**
     * Gets hashcode for this object.
     * @return hash code.
     */
    @Override
    public int hashCode() {
        return file.hashCode();
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        return getName();
    }

}
