/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Deployer.java 1026 2006-08-04 14:54:10Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployer;

import static org.objectweb.easybeans.util.url.URLUtils.fileToURL2;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.jar.JarFile;

import javax.enterprise.deploy.shared.ModuleType;

import org.objectweb.easybeans.api.EZBArchive;
import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.EZBContainerException;
import org.objectweb.easybeans.container.archive.ArchiveManager;
import org.objectweb.easybeans.loader.EasyBeansClassLoader;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.persistence.PersistenceUnitManager;
import org.objectweb.easybeans.persistence.xml.PersistenceXmlFileAnalyzer;
import org.objectweb.easybeans.persistence.xml.PersistenceXmlFileAnalyzerException;
import org.objectweb.easybeans.server.Embedded;
import org.objectweb.easybeans.util.files.FileUtils;
import org.objectweb.easybeans.util.files.FileUtilsException;
import org.objectweb.easybeans.util.url.URLUtilsException;

/**
 * Deployer allowing deploy/undeploy functions on this container.
 * @author Florent Benoit
 */
public final class Deployer implements IDeployer {

    /**
     * Embedded server linked to this deployer.
     */
    private Embedded embedded = null;

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Deployer.class);

    /**
     * Folder to create in tmp folder.
     */
    private static final String DEFAULT_FOLDER = "EasyBeans-Deployer";

    /**
     * Build a Deployer for the given Embedded instance.
     * @param embedded the Server instance.
     */
    public Deployer(final Embedded embedded) {
        this.embedded = embedded;
    }

    /**
     * Deploy a file on the EJB3 container.
     * @param typeparam type of the file (EJB, WAR, EAR, RAR, CAR) (ModuleType
     *        of JSR88)
     * @param bfile bytes array of the file
     * @param filename basename of the file to be deployed
     * @return the path of the deployed file
     */
    public String deployFile(final Integer typeparam, final java.lang.Byte[] bfile, final String filename) {
        logger.info("Trying to deploy file with param {0}, filename = {1}", typeparam, filename);

        int type = typeparam.intValue();

        String returnedPath = null;

        File rootFolder = new File(System.getProperty("java.io.tmpdir") + File.separator + DEFAULT_FOLDER);
        rootFolder.mkdirs();

        // Create file in tmp folder
        File tmpFolder =  new File(rootFolder, "TMP");
        tmpFolder.mkdirs();
        File file =  new File(tmpFolder, filename);

        FileOutputStream out = null;
        try {
            out = new FileOutputStream(file);
        } catch (FileNotFoundException e) {
            throw new IllegalStateException("Cannot build an outputstream on file '" + file + "'., e");
        }
        byte[] bfileused = new byte[bfile.length];
        for (int i = 0; i < bfile.length; i++) {
            bfileused[i] = bfile[i].byteValue();
        }
        try {
            out.write(bfileused);
        } catch (IOException e) {
            throw new IllegalStateException("Cannot write byte in outputstream", e);
        }
        try {
            out.close();
        } catch (IOException e) {
            throw new IllegalStateException("Cannot close outpustream", e);
        }

        if (type == ModuleType.EAR.getValue()) {
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(file);
            } catch (IOException e) {
                throw new IllegalStateException("Cannot build a Jar file on file '" + file + "'.", e);
            }
            File appsFolder = new File(rootFolder, "APPS");
            appsFolder.mkdirs();
            File dest = new File(appsFolder, filename);
            try {
                FileUtils.unpack(jarFile, dest);
            } catch (FileUtilsException e) {
                throw new IllegalStateException("Cannot unpack file '" + jarFile + "'.", e);
            }
            logger.info("Unpack of ear {0} in folder {1}", file, dest);
            try {
                jarFile.close();
            } catch (IOException e) {
                throw new IllegalStateException("Cannot close jarFile '" + jarFile + "'.", e);
            }
            returnedPath = dest.getAbsolutePath();
        }

        if (returnedPath == null) {
            throw new IllegalStateException("Returned path should not be null");
        }
        logger.info("Returning path = {0}", returnedPath);
        return returnedPath;
    }

    /**
     * Deploy an application packaged in a EAR file.
     * @param fileName the name of the file to deploy
     */
    public void deployEar(final String fileName) {
        logger.info("Deploying ear {0}", fileName);

        // Try to find ejb-jar.
        File earFile = new File(fileName);
        if (!earFile.isDirectory()) {
            throw new IllegalArgumentException("Handle only EAR directories");
        }

        // Init libraries list.
        List<File> libraries = new ArrayList<File>();

        // Search files of the directory (to find an ejb-jar)
        List<File> ejbFiles = new ArrayList<File>();
        File[] files = earFile.listFiles();
        if (files != null) {
            for (File f : files) {
                if (f.getName().toLowerCase().endsWith("_ejb.jar")) {
                    ejbFiles.add(f);
                }

                // lib folder in the ear ?
                if (f.isDirectory() && f.getName().equals("lib")) {
                    // Add all jars found in this directory.
                    File[] libs = f.listFiles();
                    if (libs != null) {
                        for (File lib : libs) {
                        libraries.add(lib);
                        }
                    }
                }
            }
        }

        // build common classloader with EJB and libraries
        URL[] urls = new URL[libraries.size() + ejbFiles.size()];
        int u = 0;
        for (File library : libraries) {
            try {
                urls[u++] = fileToURL2(library);
            } catch (URLUtilsException e) {
                throw new IllegalStateException("Cannot transform file '" + library + "' into an URL.", e);
            }
        }
        for (File ejb : ejbFiles) {
            try {
                urls[u++] = fileToURL2(ejb);
            } catch (URLUtilsException e) {
                throw new IllegalStateException("Cannot transform file '" + ejb + "' into an URL.", e);
            }
        }
        ClassLoader appClassLoader = new EasyBeansClassLoader(urls, Thread.currentThread().getContextClassLoader());

        // Analyze libraries to detect persistence archive (only once for now and for all libraries)
        PersistenceUnitManager persistenceUnitManager = null;
        for (File library : libraries) {
            PersistenceUnitManager builtPersistenceUnitManager = null;
            try {
                EZBArchive archive = ArchiveManager.getInstance().getArchive(library);
                builtPersistenceUnitManager = PersistenceXmlFileAnalyzer.analyzePersistenceXmlFile(archive, appClassLoader);
            } catch (PersistenceXmlFileAnalyzerException e) {
                throw new IllegalStateException("Failure when analyzing the persistence.xml file", e);
            }
            if (persistenceUnitManager != null) {
                throw new IllegalStateException("There was an existing persistence.xml in another library of this EAR."
                        + " This is not supported.");
            }
            persistenceUnitManager = builtPersistenceUnitManager;
        }


        if (ejbFiles.size() == 0) {
            throw new IllegalStateException("No EJB-JAR file was found in the ear file.");
        }

        // Create containers
        List<EZBContainer> containers = new ArrayList<EZBContainer>();
        for (File ejbFile : ejbFiles) {
            containers.add(createContainer(ejbFile.getAbsolutePath()));
        }

        // Configure containers
        for (EZBContainer container : containers) {
            // Set the classloader that needs to be used
            container.setClassLoader(appClassLoader);
            // Add persistence context found
            container.setPersistenceUnitManager(persistenceUnitManager);
        }



        // Start containers
        for (EZBContainer container : containers) {
            try {
                container.start();
            } catch (EZBContainerException e) {
                logger.error("Cannot start container {0}", container.getName(), e);
            }
        }

    }

    /**
     * Deploy an EJB packaged in a JAR file.
     * @param fileName the name of the file to deploy
     */
    public void deployEjbJar(final String fileName) {
        logger.info("Deploying EJB JAR {0}", fileName);
        EZBContainer container = createContainer(fileName);
        try {
            container.start();
        } catch (EZBContainerException e) {
            logger.error("Cannot start container {0}", container.getName(), e);
        }
    }

    /**
     * Create a new container for the given fileName.
     * @param fileName the name of the file for which a container needs to be created.
     * @return an instance of a container.
     */
    private EZBContainer createContainer(final String fileName) {
        File f = new File(fileName);
        EZBArchive archive = ArchiveManager.getInstance().getArchive(f);
        return embedded.createContainer(archive);
    }

}
