/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AnnotationDeploymentAnalyzer.java 925 2006-07-25 12:37:03Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.analyzer;

import java.net.URL;
import java.net.URLConnection;
import java.util.Iterator;

import org.objectweb.asm.ClassReader;
import org.objectweb.easybeans.api.EZBArchive;
import org.objectweb.easybeans.api.EZBArchiveException;
import org.objectweb.easybeans.deployment.annotations.exceptions.AnalyzerException;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * This class finds the annotated class and fill metadata class.
 * @author Florent Benoit
 */
public class AnnotationDeploymentAnalyzer {

    /**
     * Archive which will be analyzed.
     */
    private EZBArchive archive = null;

    /**
     * Metadata representing the parsed ejb-jar file.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(AnnotationDeploymentAnalyzer.class);

    /**
     * ASM visitor used to visit classes.
     */
    private ScanClassVisitor scanVisitor = null;

    /**
     * Constructor.<br>
     * Archive which will be used when analyzing.
     * @param archive the archive to analyze.
     */
    public AnnotationDeploymentAnalyzer(final EZBArchive archive) {
        this.archive = archive;
        this.ejbJarAnnotationMetadata = new EjbJarAnnotationMetadata();
        scanVisitor = new ScanClassVisitor(ejbJarAnnotationMetadata);
    }

    /**
     * Analyzes the archive and fill metadata struct.
     * @throws AnalyzerException if analyze of archive fails
     */
    public void analyze() throws AnalyzerException {

        Iterator<URL> iterator;
        try {
            iterator = archive.getResources();
            // Scan all resources
            while (iterator.hasNext()) {
                URL url = iterator.next();
                // only .class
                if (url.getPath().toLowerCase().endsWith(".class")) {
                    try {
                        URLConnection urlConnection = url.openConnection();
                        urlConnection.setDefaultUseCaches(false);
                        new ClassReader(urlConnection.getInputStream()).accept(scanVisitor, false);
                    } catch (Exception ioe) {
                        throw new AnalyzerException("Error while analyzing file entry '" + url + "' in archive '"
                                + archive.getName() + "'", ioe);
                    }
                }
            }

        } catch (EZBArchiveException e) {
            throw new AnalyzerException("Error while analyzing archive '" + archive.getName() + "'", e);
        } finally {
            archive.close();
        }

    }


    /**
     * @return struct (metadata) filled by this analyzer
     */
    public EjbJarAnnotationMetadata getEjbJarAnnotationMetadata() {
        return ejbJarAnnotationMetadata;
    }

}
