/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EJB3DeploymentDescLoader.java 1248 2006-12-12 13:18:47Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.xml.parsing;

import static org.objectweb.easybeans.deployment.xml.struct.common.LifeCycleCallback.POST_ACTIVATE;
import static org.objectweb.easybeans.deployment.xml.struct.common.LifeCycleCallback.POST_CONSTRUCT;
import static org.objectweb.easybeans.deployment.xml.struct.common.LifeCycleCallback.PRE_DESTROY;
import static org.objectweb.easybeans.deployment.xml.struct.common.LifeCycleCallback.PRE_PASSIVATE;

import java.net.URL;
import java.util.List;

import org.objectweb.easybeans.deployment.annotations.impl.JApplicationException;
import org.objectweb.easybeans.deployment.xml.struct.AbsBean;
import org.objectweb.easybeans.deployment.xml.struct.AssemblyDescriptor;
import org.objectweb.easybeans.deployment.xml.struct.EJB3;
import org.objectweb.easybeans.deployment.xml.struct.EnterpriseBeans;
import org.objectweb.easybeans.deployment.xml.struct.Interceptor;
import org.objectweb.easybeans.deployment.xml.struct.InterceptorBinding;
import org.objectweb.easybeans.deployment.xml.struct.Interceptors;
import org.objectweb.easybeans.deployment.xml.struct.RemoveMethod;
import org.objectweb.easybeans.deployment.xml.struct.Session;
import org.objectweb.easybeans.deployment.xml.struct.common.AbsEJBRef;
import org.objectweb.easybeans.deployment.xml.struct.common.AbsEnvironment;
import org.objectweb.easybeans.deployment.xml.struct.common.AbsResourceGroup;
import org.objectweb.easybeans.deployment.xml.struct.common.AroundInvoke;
import org.objectweb.easybeans.deployment.xml.struct.common.EJBLocalRef;
import org.objectweb.easybeans.deployment.xml.struct.common.EJBRef;
import org.objectweb.easybeans.deployment.xml.struct.common.EnvEntry;
import org.objectweb.easybeans.deployment.xml.struct.common.InjectionTarget;
import org.objectweb.easybeans.deployment.xml.struct.common.LifeCycleCallback;
import org.objectweb.easybeans.deployment.xml.struct.common.MethodDD;
import org.objectweb.easybeans.deployment.xml.struct.common.ResourceEnvRef;
import org.objectweb.easybeans.deployment.xml.struct.common.ResourceRef;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.util.xml.DocumentParser;
import org.objectweb.easybeans.util.xml.DocumentParserException;
import org.objectweb.easybeans.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class analyzes the EJB3 deployment descriptor and fill a struct with
 * getters.
 * @author Florent Benoit
 */
public final class EJB3DeploymentDescLoader {

    /**
     * Persistence namespace (Java EE).
     */
    private static final String JAVAEE_NS = "http://java.sun.com/xml/ns/javaee";

    /**
     * Persistence namespace (J2EE).
     */
    private static final String J2EE_NS = "http://java.sun.com/xml/ns/j2ee";

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(EJB3DeploymentDescLoader.class);

    /**
     * Validating with schema ?
     */
    private static boolean validating = true;

    /**
     * EJB3 instance object that is the result of the parsing.
     */
    private EJB3 ejb3 = null;

    /**
     * URL of the file being analyzed.
     */
    private URL url = null;

    /**
     * Namespace to use when analyzing the XML file. It depends if the archive is J2EE or Java EE module.
     */
    private String xmlns = null;

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the EJB Xml file.
     */
    private EJB3DeploymentDescLoader(final URL url) {
        this.url = url;
        this.ejb3 = new EJB3();
    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static EJB3 loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        EJB3DeploymentDescLoader ejbDeploymentDescLoader = new EJB3DeploymentDescLoader(url);
        ejbDeploymentDescLoader.parse();
        return ejbDeploymentDescLoader.getEJB3();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(url, validating, new EJB3EntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <ejb-jar>
        Element ejbJarRootElement = document.getDocumentElement();

        // get the namespace of this element
        String ns = ejbJarRootElement.getNamespaceURI();
        if (JAVAEE_NS.equals(ns)) {
            xmlns = JAVAEE_NS;
        } else if (J2EE_NS.equals(ns)) {
            xmlns = J2EE_NS;
        } else {
            throw new ParsingException("Unsupported XML namespace: " + ns);
        }

        // enterprise-beans
        NodeList enterprisebeansList = ejbJarRootElement.getElementsByTagNameNS(xmlns, EnterpriseBeans.NAME);
        if (enterprisebeansList.getLength() == 1) {
            Element enterpriseBeansElement = (Element) enterprisebeansList.item(0);

            // Create struct
            EnterpriseBeans enterpriseBeans = new EnterpriseBeans();
            ejb3.setEnterpriseBeans(enterpriseBeans);

            // Analyze beans
            analyzeBeans(enterpriseBeansElement, enterpriseBeans);

        }

        // interceptors
        NodeList interceptorsList = ejbJarRootElement.getElementsByTagNameNS(xmlns, Interceptors.NAME);
        if (interceptorsList.getLength() == 1) {
            Element interceptorsElement = (Element) interceptorsList.item(0);

            // Create struct
            Interceptors interceptors = new Interceptors();
            ejb3.setInterceptors(interceptors);

            // Analyze the interceptors element
            analyzeInterceptorsElement(interceptorsElement, interceptors);
        }

        // assembly-descriptor
        NodeList assemblyDescriptorList = ejbJarRootElement.getElementsByTagNameNS(xmlns, AssemblyDescriptor.NAME);
        if (assemblyDescriptorList.getLength() == 1) {
            Element assemblyDescriptorElement = (Element) assemblyDescriptorList.item(0);

            // Create struct
            AssemblyDescriptor assemblyDescriptor = new AssemblyDescriptor();
            ejb3.setAssemblyDescriptor(assemblyDescriptor);

            // Analyze the application-exception element
            analyzeAssemblyDescriptor(assemblyDescriptorElement, assemblyDescriptor);
        }

    }

    /**
     * Analyze common stuff of beans.
     * @param enterpriseBeansElement the dom element.
     * @param enterpriseBeans the structure representing object.
     */
    private void analyzeBeans(final Element enterpriseBeansElement, final EnterpriseBeans enterpriseBeans) {
        // Get session beans
        NodeList sessionBeansList = enterpriseBeansElement.getElementsByTagNameNS(xmlns, Session.NAME);

        // Loop on session beans
        for (int i = 0; i < sessionBeansList.getLength(); i++) {
            Element sessionElement = (Element) sessionBeansList.item(i);

            // Build instance that is created.
            Session session = new Session();

            enterpriseBeans.addSession(session);

            // First, analyze common things
            analyzeSessionBean(sessionElement, session);
        }

    }

    /**
     * Analyze common stuff of a bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeBean(final Element beanElement, final AbsBean bean) {
        // Get ejb-name
        String ejbName = XMLUtils.getStringValueElement(xmlns, beanElement, "ejb-name");
        bean.setEjbName(ejbName);

        // Get ejb-class
        String ejbClass = XMLUtils.getStringValueElement(xmlns, beanElement, "ejb-class");
        bean.setEjbClass(ejbClass);

        // Analyze common things
        analyzeEnvironement(beanElement, bean);
    }

    /**
     * Analyze XML of a session bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeSessionBean(final Element beanElement, final Session bean) {
        analyzeBean(beanElement, bean);

        // Get business-remote
        List<String> businessRemoteList = XMLUtils.getStringListValueElement(xmlns, beanElement, "business-remote");
        for (String businessRemote : businessRemoteList) {
            bean.addBusinessRemote(businessRemote);
        }

        // Get business-local
        List<String> businessLocalList = XMLUtils.getStringListValueElement(xmlns, beanElement, "business-local");
        for (String businessLocal : businessLocalList) {
            bean.addBusinessLocal(businessLocal);
        }

        // Mapped Name
        String mappedName = XMLUtils.getChildStringValueForElement(beanElement, "mapped-name");
        if (mappedName != null) {
            bean.setMappedName(mappedName);
        }

        // Remove Methods
        NodeList removeMethodList = beanElement.getElementsByTagNameNS(xmlns, RemoveMethod.NAME);

        // Loop on the remove-method
        for (int i = 0; i < removeMethodList.getLength(); i++) {
            Element removeMethodElement = (Element) removeMethodList.item(i);

            RemoveMethod removeMethod = new RemoveMethod();
            bean.addRemoveMethod(removeMethod);

            // Analyze method
            NodeList beanMethodList = removeMethodElement.getElementsByTagNameNS(xmlns, "bean-method");
            removeMethod.setMethod(getMethodFromElement((Element) beanMethodList.item(0)));

            // Retain if exception
            String retainIfExceptionStr = XMLUtils.getStringValueElement(xmlns, removeMethodElement,
                    "retain-if-exception");
            if (retainIfExceptionStr != null) {
                // convert into boolean value
                boolean retainIfException = false;
                retainIfException = Boolean.parseBoolean(retainIfExceptionStr);
                removeMethod.setRetainIfException(retainIfException);
            }
        }

        // Get remote home
        String remoteHome = XMLUtils.getChildStringValueForElement(beanElement, "home");
        bean.setRemoteHome(remoteHome);

        // Get Local home
        String localHome = XMLUtils.getChildStringValueForElement(beanElement, "local-home");
        bean.setLocalHome(localHome);

        // Get session-type
        String sessionType = XMLUtils.getStringValueElement(xmlns, beanElement, "session-type");
        bean.setSessionType(sessionType);

        // Get transaction-type
        String transactionType = XMLUtils.getStringValueElement(xmlns, beanElement, "transaction-type");
        bean.setTransactionType(transactionType);

    }


    /**
     * Gets the methodDD from the given dom element.
     * @param methodElement the dom element to analyze
     * @return the given method object
     */
    private MethodDD getMethodFromElement(final Element methodElement) {
        // create object
        MethodDD method = new MethodDD();

        // Analyze name
        String methodName = XMLUtils.getStringValueElement(xmlns, methodElement, "method-name");
        method.setName(methodName);

        // Params
        NodeList methodParamsList = methodElement.getElementsByTagNameNS(xmlns, "method-params");
        if (methodParamsList != null && methodParamsList.getLength() == 1) {
            // first item is the method-params element
            Element methodParamsElement = (Element) methodParamsList.item(0);

            List<String> paramList = XMLUtils.getStringListValueElement(xmlns, methodParamsElement,
            "method-param");
            for (String param : paramList) {
                method.addParam(param);
            }

        }
        return method;
    }

    /**
     * Analyze the environment (could be a bean, servlet, etc).
     * @param envElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeEnvironement(final Element envElement, final AbsEnvironment bean) {
        analyzeLifeCycle(envElement, bean, POST_CONSTRUCT);
        analyzeLifeCycle(envElement, bean, PRE_DESTROY);
        analyzeLifeCycle(envElement, bean, PRE_PASSIVATE);
        analyzeLifeCycle(envElement, bean, POST_ACTIVATE);
        analyzeAroundInvoke(envElement, bean);
        analyzeEnvEnvtry(envElement, bean);
        analyzeEJBRef(envElement, bean);
        analyzeEJBLocalRef(envElement, bean);
        analyzeResourceRef(envElement, bean);
        analyzeResourceEnvRef(envElement, bean);
    }

    /**
     * Analyze the ejb-ref/local-ref element and find the sub-elements.
     * @param refElement given XML element which contains ejb-ref-name, ejb-ref-type, etc.
     * @param ref the struct object representing the XML element.
     */
    private void analyzeCommonEJBRef(final Element refElement, final AbsEJBRef ref) {
        // ejb-ref-name
        String ejbRefName = XMLUtils.getStringValueElement(xmlns, refElement, "ejb-ref-name");
        ref.setEjbRefName(ejbRefName);

        // ejb-ref-type
        String ejbRefType = XMLUtils.getStringValueElement(xmlns, refElement, "ejb-ref-type");
        ref.setEjbRefType(ejbRefType);

        // mapped-name
        String mappedName = XMLUtils.getStringValueElement(xmlns, refElement, "mapped-name");
        ref.setMappedName(mappedName);

        // ejb-link
        String ejbLink = XMLUtils.getStringValueElement(xmlns, refElement, "ejb-link");
        ref.setEjbLink(ejbLink);

        // Analyze injection-target
        analyzeInjectionTarget(refElement, ref);
    }

    /**
     * Analyze the injection-target element and add them to the given resource
     * group object used by ejb-ref, ejb-local-ref, resource-ref, etc.
     * @param refElement given XML element which contains a resource element.
     * @param resourceGroup the struct object representing the XML element.
     */
    private void analyzeInjectionTarget(final Element refElement, final AbsResourceGroup resourceGroup) {
        // injection target
        NodeList injectionTargetList = refElement.getElementsByTagNameNS(xmlns, "injection-target");
        for (int j = 0; j < injectionTargetList.getLength(); j++) {
            Element injectionTargetElement = (Element) injectionTargetList.item(j);

            // Build Object and add it
            InjectionTarget injectionTarget = new InjectionTarget();

            // Get class name
            String className = XMLUtils.getStringValueElement(xmlns, injectionTargetElement, "injection-target-class");
            injectionTarget.setClassname(className);

            // Get target
            String targetName = XMLUtils.getStringValueElement(xmlns, injectionTargetElement, "injection-target-name");
            injectionTarget.setTargetName(targetName);

            resourceGroup.addInjectionTarget(injectionTarget);
        }

    }

    /**
     * Analyze the environment element and find the ejb-ref element.
     * @param envElement given XML element which contains ejb-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeEJBRef(final Element envElement, final AbsEnvironment env) {
        NodeList ejbRefList = envElement.getElementsByTagNameNS(xmlns, EJBRef.NAME);

        // Loop on this list
        for (int i = 0; i < ejbRefList.getLength(); i++) {
            Element ejbRefElement = (Element) ejbRefList.item(i);
            EJBRef ejbRef = new EJBRef();

            analyzeCommonEJBRef(ejbRefElement, ejbRef);

            // remote
            String remote = XMLUtils.getStringValueElement(xmlns, ejbRefElement, "remote");
            ejbRef.setRemote(remote);

            // home
            String home = XMLUtils.getStringValueElement(xmlns, ejbRefElement, "home");
            ejbRef.setHome(home);

            // add ejb-ref in environment element
            env.addEJBRef(ejbRef);
        }
    }

    /**
     * Analyze the environment element and find the resource-ref element.
     * @param envElement given XML element which contains resource-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeResourceRef(final Element envElement, final AbsEnvironment env) {
        NodeList resourceRefList = envElement.getElementsByTagNameNS(xmlns, ResourceRef.NAME);

        // Loop on this list
        for (int i = 0; i < resourceRefList.getLength(); i++) {
            Element resourceRefElement = (Element) resourceRefList.item(i);
            ResourceRef resourceRef = new ResourceRef();

            // name
            String resRefName = XMLUtils.getStringValueElement(xmlns, resourceRefElement, "res-ref-name");
            resourceRef.setResRefName(resRefName);

            // type
            String resRefType = XMLUtils.getStringValueElement(xmlns, resourceRefElement, "res-ref-type");
            resourceRef.setResRefType(resRefType);

            // Analyze injection-target
            analyzeInjectionTarget(envElement, resourceRef);

            // add resource-ref in environment element
            env.addResourceRef(resourceRef);
        }
    }

    /**
     * Analyze the environment element and find the resource-env-ref element.
     * @param envElement given XML element which contains resource-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeResourceEnvRef(final Element envElement, final AbsEnvironment env) {
        NodeList resourceEnvRefList = envElement.getElementsByTagNameNS(xmlns, ResourceEnvRef.NAME);

        // Loop on this list
        for (int i = 0; i < resourceEnvRefList.getLength(); i++) {
            Element resourceRefElement = (Element) resourceEnvRefList.item(i);
            ResourceEnvRef resourceEnvRef = new ResourceEnvRef();

            // name
            String resourceEnvRefName = XMLUtils.getStringValueElement(xmlns, resourceRefElement, "resource-env-ref-name");
            resourceEnvRef.setResourceEnvRefName(resourceEnvRefName);

            // type
            String resourceEnvRefType = XMLUtils.getStringValueElement(xmlns, resourceRefElement, "resource-env-ref-type");
            resourceEnvRef.setResourceEnvRefType(resourceEnvRefType);

            // Analyze injection-target
            analyzeInjectionTarget(envElement, resourceEnvRef);

            // add resource-ref in environment element
            env.addResourceEnvRef(resourceEnvRef);
        }
    }

    /**
     * Analyze the environment element and find the ejb-local-ref element.
     * @param envElement given XML element which contains ejb-local-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeEJBLocalRef(final Element envElement, final AbsEnvironment env) {
        NodeList ejbLocalRefList = envElement.getElementsByTagNameNS(xmlns, EJBLocalRef.NAME);

        // Loop on this list
        for (int i = 0; i < ejbLocalRefList.getLength(); i++) {
            Element ejbRefElement = (Element) ejbLocalRefList.item(i);
            EJBLocalRef ejbLocalRef = new EJBLocalRef();

            analyzeCommonEJBRef(ejbRefElement, ejbLocalRef);

            // local
            String local = XMLUtils.getStringValueElement(xmlns, ejbRefElement, "local");
            ejbLocalRef.setLocal(local);

            // local-home
            String localHome = XMLUtils.getStringValueElement(xmlns, ejbRefElement, "local-home");
            ejbLocalRef.setLocalHome(localHome);

            // add ejb-local-ref in environment element
            env.addEJBLocalRef(ejbLocalRef);
        }
    }

    /**
     * Analyze the lifecycle elements.
     * @param envElement the dom element.
     * @param env the structure representing object.
     */
    private void analyzeAroundInvoke(final Element envElement, final AbsEnvironment env) {
        // Get the around-invoke elements
        NodeList aroundInvokeList = envElement.getElementsByTagNameNS(xmlns, "around-invoke");

        // Loop on the around-invoke
        for (int i = 0; i < aroundInvokeList.getLength(); i++) {
            Element aroundInvokeElement = (Element) aroundInvokeList.item(i);

            // Build instance that is created.
            AroundInvoke aroundInvoke = new AroundInvoke();
            env.addAroundInvoke(aroundInvoke);

            // lifecycle-callback-class
            String className = XMLUtils.getStringValueElement(xmlns, aroundInvokeElement, "class");
            aroundInvoke.setClassName(className);

            // lifecycle-callback-method
            String method = XMLUtils.getStringValueElement(xmlns, aroundInvokeElement, "method-name");
            aroundInvoke.setMethodName(method);

        }

    }

    /**
     * Analyze the lifecycle elements.
     * @param envElement the dom element.
     * @param env the structure representing object.
     * @param lifecycleName the name of lifecycle (post-construct, etc)
     */
    private void analyzeLifeCycle(final Element envElement, final AbsEnvironment env, final String lifecycleName) {
        // Get the lifecycle elements
        NodeList lifecycleList = envElement.getElementsByTagNameNS(xmlns, lifecycleName);

        // Loop on the lifecycle
        for (int i = 0; i < lifecycleList.getLength(); i++) {
            Element lifeCycleElement = (Element) lifecycleList.item(i);

            // Build instance that is created.
            LifeCycleCallback lifecycleCallback = new LifeCycleCallback();
            if (POST_CONSTRUCT.equals(lifecycleName)) {
                env.addPostConstructCallback(lifecycleCallback);
            } else if (PRE_DESTROY.equals(lifecycleName)) {
                env.addPreDestroyCallback(lifecycleCallback);
            } else if (PRE_PASSIVATE.equals(lifecycleName)) {
                env.addPrePassivateCallback(lifecycleCallback);
            } else if (POST_ACTIVATE.equals(lifecycleName)) {
                env.addPostActivateCallback(lifecycleCallback);
            } else {
                throw new IllegalArgumentException("Unknown lifecycle named '" + lifecycleName + "'.");
            }

            // lifecycle-callback-class
            String className = XMLUtils.getStringValueElement(xmlns, lifeCycleElement, "lifecycle-callback-class");
            lifecycleCallback.setLifecycleCallbackClass(className);

            // lifecycle-callback-method
            String method = XMLUtils.getStringValueElement(xmlns, lifeCycleElement, "lifecycle-callback-method");
            lifecycleCallback.setMethodName(method);

        }

    }

    /**
     * Analyze the env-entry element.
     * @param envElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeEnvEnvtry(final Element envElement, final AbsEnvironment bean) {
        // Get env-entry
        NodeList envEntryList = envElement.getElementsByTagNameNS(xmlns, EnvEntry.NAME);

        // Loop on env entry
        for (int i = 0; i < envEntryList.getLength(); i++) {
            Element envEntryElement = (Element) envEntryList.item(i);

            // Build instance that is created.
            EnvEntry envEntry = new EnvEntry();
            bean.addEnvEntry(envEntry);

            // Env-entry name
            String envEntryName = XMLUtils.getStringValueElement(xmlns, envEntryElement, "env-entry-name");
            envEntry.setEnvEntryName(envEntryName);

            // Env-entry type
            String envEntryType = XMLUtils.getStringValueElement(xmlns, envEntryElement, "env-entry-type");
            envEntry.setEnvEntryType(envEntryType);

            // Env-entry value
            String envEntryValue = XMLUtils.getStringValueElement(xmlns, envEntryElement, "env-entry-value");
            envEntry.setEnvEntryValue(envEntryValue);
        }
    }

    /**
     * Analyze the interceptors element of an EJB-JAR.
     * @param interceptorsElement the dom element.
     * @param interceptors the structure representing object.
     */
    private void analyzeInterceptorsElement(final Element interceptorsElement, final Interceptors interceptors) {
        // Get interceptor list elements
        NodeList interceptorList = interceptorsElement.getElementsByTagNameNS(xmlns, Interceptor.NAME);

        // Loop on <interceptor> elements
        for (int i = 0; i < interceptorList.getLength(); i++) {
            Element interceptorElement = (Element) interceptorList.item(i);

            Interceptor interceptor = new Interceptor();
            interceptors.addInterceptor(interceptor);

            // Extract interceptor class
            String interceptorClass = XMLUtils.getStringValueElement(xmlns, interceptorElement, "interceptor-class");
            interceptor.setInterceptorClass(interceptorClass);

            // Analyze the object
            analyzeEnvironement(interceptorElement, interceptor);
        }
    }

    /**
     * Analyze the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeAssemblyDescriptor(final Element assemblyDescriptorElement,
            final AssemblyDescriptor assemblyDescriptor) {
        analyzeApplicationException(assemblyDescriptorElement, assemblyDescriptor);

        analyzeInterceptorBinding(assemblyDescriptorElement, assemblyDescriptor);
    }

    /**
     * Analyze the interceptor binding of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeInterceptorBinding(final Element assemblyDescriptorElement,
            final AssemblyDescriptor assemblyDescriptor) {
        // Get interceptor-binding list
        NodeList interceptorBindingList = assemblyDescriptorElement.getElementsByTagNameNS(xmlns, InterceptorBinding.NAME);

        // Loop on interceptor-binding
        for (int i = 0; i < interceptorBindingList.getLength(); i++) {
            Element interceptorBindingElement = (Element) interceptorBindingList.item(i);

            // Build struct and set it in the assembly descriptor
            InterceptorBinding interceptorBinding = new InterceptorBinding();
            assemblyDescriptor.addInterceptorBinding(interceptorBinding);

            // name
            String ejbName = XMLUtils.getStringValueElement(xmlns, interceptorBindingElement, "ejb-name");
            interceptorBinding.setEjbName(ejbName);

            // interceptor-order
            NodeList interceptorOrderList = interceptorBindingElement.getElementsByTagNameNS(xmlns, "interceptor-order");
            if (interceptorOrderList != null && interceptorOrderList.getLength() == 1) {
                // first item is the interceptor-order element
                Element interceptorOrderElement = (Element) interceptorOrderList.item(0);
                List<String> interceptorOrderClassList = XMLUtils.getStringListValueElement(xmlns, interceptorOrderElement,
                        "interceptor-class");
                for (String interceptorClass : interceptorOrderClassList) {
                    interceptorBinding.addOrderedInterceptorClass(interceptorClass);
                }
            } else {
                // interceptor-class (as there is no interceptor-order element)
                List<String> interceptorClassList = XMLUtils.getStringListValueElement(xmlns, interceptorBindingElement,
                        "interceptor-class");
                for (String interceptorClass : interceptorClassList) {
                    interceptorBinding.addInterceptorClass(interceptorClass);
                }
            }

            // method
            NodeList methodList = interceptorBindingElement.getElementsByTagNameNS(xmlns, "method");
            if (methodList.getLength() > 0) {
                interceptorBinding.setMethod(getMethodFromElement((Element) methodList.item(0)));
            }

            // exclude-class-interceptors
            String excludeClassInterceptorsStr = XMLUtils.getStringValueElement(xmlns, interceptorBindingElement,
                    "exclude-class-interceptors");
            if (excludeClassInterceptorsStr != null) {
                // convert into boolean value
                boolean excludeClassInterceptors = false;
                excludeClassInterceptors = Boolean.parseBoolean(excludeClassInterceptorsStr);
                interceptorBinding.setExcludeClassInterceptors(excludeClassInterceptors);
            }

            // exclude-default-interceptors
            String excludeDefaultInterceptorsStr = XMLUtils.getStringValueElement(xmlns, interceptorBindingElement,
                    "exclude-default-interceptors");
            if (excludeDefaultInterceptorsStr != null) {
                // convert into boolean value
                boolean excludeDefaultInterceptors = false;
                excludeDefaultInterceptors = Boolean.parseBoolean(excludeDefaultInterceptorsStr);
                interceptorBinding.setExcludeDefaultInterceptors(excludeDefaultInterceptors);
            }

        }
    }

    /**
     * Analyze the application Exception of the assembly descriptor of an
     * EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeApplicationException(final Element assemblyDescriptorElement,
            final AssemblyDescriptor assemblyDescriptor) {
        // Get application-exception list
        NodeList applicationExceptionList = assemblyDescriptorElement.getElementsByTagNameNS(xmlns,
                JApplicationException.NAME);

        // Loop on application-exception
        for (int i = 0; i < applicationExceptionList.getLength(); i++) {
            Element applicationExceptionElement = (Element) applicationExceptionList.item(i);

            // name
            String className = XMLUtils.getStringValueElement(xmlns, applicationExceptionElement, "exception-class");

            // rollback
            String rollbackStr = XMLUtils.getStringValueElement(xmlns, applicationExceptionElement, "rollback");

            // convert into boolean value
            boolean rollback = false;
            if (rollbackStr != null) {
                rollback = Boolean.parseBoolean(rollbackStr);
            }

            // Build instance that is created.
            JApplicationException applicationException = new JApplicationException(className, rollback);
            assemblyDescriptor.addApplicationException(applicationException);
        }
    }

    /**
     * @return the object resulting of the XML parsing.
     */
    public EJB3 getEJB3() {
        return ejb3;
    }

}
